
#include "main.h"

volatile unsigned long int timerMilliseconds = 0;		// globaler Zhler fr die Zeit seit Systemstart in Millisekunden
volatile unsigned char colorData[12][3] = {{0}};	// Farbwerte fr die LEDs (in RGB von 0-7), wird vom Timer-Interrupt ausgewertet
unsigned char colorDataScript[12][3];

const prog_char versionNumber[] = "2.4";	// Software-Versionsnummer
const prog_char textAn[] = "an";
const prog_char textAus[] = "aus";
const prog_char ppmmodeText[] = "\nPPM-Mode";
const prog_char errorText[] = "error\n";

Configuration configuration;
Configuration EEMEM EEPromConfiguration = {TESTMODE, AKKU_SPANNUNG_LEER, AKKU_SPANNUNG_VOLL, 0, DEFAULT_BLINK_INTERVAL, {DEFAULT_AKKU_COLOR_EMPTY_RED, DEFAULT_AKKU_COLOR_EMPTY_GREEN, DEFAULT_AKKU_COLOR_EMPTY_BLUE}, {DEFAULT_AKKU_COLOR_CRITICAL_RED, DEFAULT_AKKU_COLOR_CRITICAL_GREEN, DEFAULT_AKKU_COLOR_CRITICAL_BLUE} , {DEFAULT_AKKU_COLOR_OK_RED, DEFAULT_AKKU_COLOR_OK_GREEN, DEFAULT_AKKU_COLOR_OK_BLUE}, 0, DEFAULT_I2C_ADDRESS, 0, 4, 0, 0, 0, 32, 62, 0, {0, 0, 0}};

unsigned short int akkuVoltageAverage = 0;	// gemittelte Akku-Spannung in Millivolt

unsigned char EEMEM EEPromDummy[512 - SCRIPT_DATA_SIZE - sizeof(Configuration)] = {0};	// Dummy, damit die Skript-Daten am Ende liegen

unsigned char EEMEM EEPromSkriptData[SCRIPT_DATA_SIZE];

unsigned short int scriptBitIndex = 0;
unsigned long int playDelay = 3000;

unsigned short int jumpBitIndex = 0;	// Position des Sprungbefehls
unsigned char jumpLoopCount = 0;		// zhlt die Durchlufe beim Sprungbefehl

unsigned long int akkuBlinkDelay = 0;

volatile unsigned char checkPPM = 0;	// Flag, ob das PPM-Signal an SCK ausgewertet werden soll
volatile unsigned char ppmLow;
volatile unsigned char ppmHigh;
volatile unsigned char ppmOk = 0;

unsigned char ppmValue = 0;


#ifdef USE_UART
unsigned char uartBuffer[UART_BUFFER_SIZE];
unsigned char uartBufferIndex = 0;
#endif

#ifdef USE_TWI_SLAVE
unsigned char twiBuffer[TWI_BUFFER_SIZE];
unsigned char twiBufferIndex = 0;
#endif


// initialisiert die Ports
void initPorts(void)
{
	// Port B
	// PB0 = BLUE
	// PB1 = RED
	// PB2 = SHIFT_CLK
	DDRB = (1 << PB0) | (1 << PB1) | (1 << PB2);
	PORTB = (1 << PB0) | (1 << PB1) | (0 << PB2);
	
	// Port A
	// PA0 = !ENABLE
	// PA1 = GREEN
	// PA2 = STORAGE_CLK
	// PA3 = V_BATTERY
	// PA4 = SCK
	// PA5 = MISO
	// PA6 = MOSI
	// PA7 = SHIFT_IN
	DDRA = (1 << PA0) | (1 << PA1) | (1 << PA2) | (0 << PA3) | (0 << PA4) | (0 << PA5) | (0 << PA6) | (1 << PA7);
	PORTA = (1 << PA0) | (1 << PA1) | (0 << PA2) | (0 << PA3) | (1 << PA4) | (1 << PA5) | (1 << PA6) | (0 << PA7);
}


// liefert das Bitmuster ab der Stelle bitIndex der Lnge bitCount zurck
unsigned short int eepromGetBits(unsigned short int *bitIndex, unsigned char bitCount)
{
	unsigned short int result = 0;
	unsigned short int localBitIndex = *bitIndex;
	*bitIndex += bitCount;
	unsigned short int byteIndex = localBitIndex >> 3;
	do
	{
		unsigned char data = eeprom_read_byte(((const unsigned char*)(&EEPromSkriptData)) + byteIndex++);
		unsigned char byteOffset = localBitIndex & 7;
		unsigned char mask = 255;
		mask = mask >> byteOffset;
		data = data & mask;
		if (bitCount >= 8-byteOffset)
		{
			result = (result << (8-byteOffset)) | data;
			bitCount -= 8-byteOffset;
			localBitIndex += 8-byteOffset;
		}
		else
		{
			result  = (result << bitCount) | (data >> (8 - byteOffset - bitCount));
			bitCount = 0;
		}
	} while (bitCount > 0);
	return result;
}


// wertet die Skriptdaten im EEProm ab der angegebenen Bitstelle aus und liefert die Dauer zurck (in ms)
unsigned short int parseEEPromData(unsigned short int *bitIndex)
{
	// Befehl auswerten
	unsigned short int tmp = eepromGetBits(bitIndex, 3);
	
	if (tmp == SCRIPT_LOOP)
	{
		unsigned char newJump = 0;
		if ((jumpBitIndex == 0) || (jumpBitIndex != *bitIndex))
		{
			// Position des Jumpbefehls merken fr Loop-Cycle-Counter
			jumpBitIndex = *bitIndex;
			newJump = 1;
			jumpLoopCount = 0;
		}
		tmp = (unsigned short int)eepromGetBits(bitIndex, 12);	// Sprungziel
		unsigned char localLoopCount = (unsigned char)eepromGetBits(bitIndex, 5);	// Loop Count
		if (newJump)
			jumpLoopCount = localLoopCount;
		else if (jumpLoopCount > 0)
			jumpLoopCount--;
		// nur springen, wenn die Anzahl der Durchlufe im Befehl 0 ist oder die Anzahl der Sprnge noch nicht durchgefhrt wurde
		if ((localLoopCount == 0) || (jumpLoopCount > 0))
			*bitIndex = tmp;
		return 0;
	}
	
	unsigned short int duration = ((unsigned short int)eepromGetBits(bitIndex, 10)) * 10;
	unsigned char rgb[3];
	
	switch (tmp)
	{
		case SCRIPT_SET_ALL_COLORS:
			// 108 Bit Daten (3 * 3 * 12)
			for (unsigned char led=0; led<12; led++)
				for (unsigned char col=0; col<3; col++)
					colorDataScript[led][col] = (unsigned char)eepromGetBits(bitIndex, 3);
		break;
		
		case SCRIPT_SCROLL_LEFT:
			// 9 Bit Daten (3 * 3)
			// Scrollen nach links
			for (unsigned char led=0; led < 11; led++)
				memcpy((unsigned char*)colorDataScript[led], (unsigned char*)colorDataScript[led+1], 3);
			for (unsigned char col=0; col<3; col++)
				colorDataScript[11][col] = (unsigned char)eepromGetBits(bitIndex, 3);
		break;
		
		case SCRIPT_SCROLL_RIGHT:
			// 9 Bit Daten (3 * 3)
			// Scrollen nach rechts
			for (unsigned char led=11; led > 0; led--)
				memcpy((unsigned char*)colorDataScript[led], (unsigned char*)colorDataScript[led-1], 3);
			for (unsigned char col=0; col<3; col++)
				colorDataScript[0][col] = (unsigned char)eepromGetBits(bitIndex, 3);
		break;
		
		case SCRIPT_ROLL_LEFT:
			// keine Daten
			memcpy(rgb, (unsigned char*)colorDataScript[0], 3);
			for (unsigned char led=0; led < 11; led++)
				memcpy((unsigned char*)colorDataScript[led], (unsigned char*)colorDataScript[led+1], 3);
			memcpy((unsigned char*)colorDataScript[11], rgb, 3);
		break;
		
		case SCRIPT_ROLL_RIGHT:
			// keine Daten
			memcpy(rgb, (unsigned char*)colorDataScript[11], 3);
			for (unsigned char led=11; led >0; led--)
				memcpy((unsigned char*)colorDataScript[led], (unsigned char*)colorDataScript[led-1], 3);
			memcpy((unsigned char*)colorDataScript[0], rgb, 3);
		break;
		
		case SCRIPT_CHANGE_LEDS:
			// Anzahl LEDs einlesen
			tmp = eepromGetBits(bitIndex, 3)+1;
			for (int i=0; i<tmp; i++)
			{
				unsigned short int pos = eepromGetBits(bitIndex, 4);
				for (unsigned char col=0; col<3; col++)
					colorDataScript[pos][col] =  (unsigned char)eepromGetBits(bitIndex, 3);
			}
		break;
		
		case SCRIPT_ALL_SAME_COLOR:
			// 9 Bit Daten (3 * 3)
			for (unsigned char i=0; i<3; i++)
				rgb[i] = (unsigned char)eepromGetBits(bitIndex, 3);
			for (unsigned char i=0; i<12; i++)
				memcpy((unsigned char*)colorDataScript[i], rgb, 3);
		break;
	}
	return duration;
}


// liest die Variablen aus dem EEProm
void readConfiguration(void)
{
	eeprom_read_block(&configuration, &EEPromConfiguration, sizeof(configuration));
	if (configuration.akkuWarnung != 1)
		configuration.akkuWarnung = 0;
	// Plausibilittscheck der Werte
	if ((configuration.akkuSpannungLeer < 5000) || (configuration.akkuSpannungVoll <= configuration.akkuSpannungLeer) || (configuration.akkuSpannungVoll > 20000))
	{
		configuration.akkuSpannungLeer = AKKU_SPANNUNG_LEER;
		configuration.akkuSpannungVoll = AKKU_SPANNUNG_VOLL;
	}
	if (configuration.akkuCellDetection != 1)
		configuration.akkuCellDetection = 0;
	if ((configuration.akkuBlinkInterval < 30) || (configuration.akkuBlinkInterval > 5000))
		configuration.akkuBlinkInterval = DEFAULT_BLINK_INTERVAL;
	
	if ((configuration.akkuFarbeBlink[0] > 7) || (configuration.akkuFarbeBlink[1] > 7) || (configuration.akkuFarbeBlink[2] > 7))
	{
		configuration.akkuFarbeBlink[0] = DEFAULT_AKKU_COLOR_EMPTY_RED;
		configuration.akkuFarbeBlink[1] = DEFAULT_AKKU_COLOR_EMPTY_GREEN;
		configuration.akkuFarbeBlink[2] = DEFAULT_AKKU_COLOR_EMPTY_BLUE;
	}
	if ((configuration.akkuFarbeKritisch[0] > 7) || (configuration.akkuFarbeKritisch[1] > 7) || (configuration.akkuFarbeKritisch[2] > 7))
	{
		configuration.akkuFarbeKritisch[0] = DEFAULT_AKKU_COLOR_CRITICAL_RED;
		configuration.akkuFarbeKritisch[1] = DEFAULT_AKKU_COLOR_CRITICAL_GREEN;
		configuration.akkuFarbeKritisch[2] = DEFAULT_AKKU_COLOR_CRITICAL_BLUE;
	}
	if ((configuration.akkuFarbeOk[0] > 7) || (configuration.akkuFarbeOk[1] > 7) || (configuration.akkuFarbeOk[2] > 7))
	{
		configuration.akkuFarbeOk[0] = DEFAULT_AKKU_COLOR_OK_RED;
		configuration.akkuFarbeOk[1] = DEFAULT_AKKU_COLOR_OK_GREEN;
		configuration.akkuFarbeOk[2]= DEFAULT_AKKU_COLOR_OK_BLUE;
	}
	
	if ((configuration.ledsCritical < 1) || (configuration.ledsCritical > 11))
		configuration.ledsCritical = DEFAULT_LEDS_CRITICAL;
	if ((configuration.I2CSlaveAddress == 0) || (configuration.I2CSlaveAddress >= 127))
		configuration.I2CSlaveAddress = DEFAULT_I2C_ADDRESS;
	if (configuration.dimValue >= 7)
		configuration.dimValue = 0;
	if (configuration.mode == IDLE)
		configuration.mode = OFF;
	if (configuration.ppmEnable > 1)
		configuration.generateSyncPulse = 0;
	else
		configuration.ppmModeSwitchOn = 0;
	for (unsigned char i=0; i<3; i++)
		configuration.ppmMode[i] &= 3;
	if (configuration.ppmMinimum < 15)
		configuration.ppmMinimum = 15;
	else if (configuration.ppmMinimum > 77)
		configuration.ppmMinimum = 77;
	if (configuration.ppmMaximum > 78)
		configuration.ppmMaximum = 78;
	if (configuration.ppmMinimum >= configuration.ppmMaximum)
		configuration.ppmMaximum = configuration.ppmMinimum+1;
	if (configuration.ppmInvert != 1)
		configuration.ppmInvert = 0;
	if (configuration.generateSyncPulse != 1)
		configuration.generateSyncPulse = 0;
	else
		PORTA &= ~(1 << PA4);	// Pin low schalten
}


// misst die Akkuspannung und liefert das Ergebnis in Millivolt zurck
unsigned short int getBatteryVoltage(void)
{
	ADCSRA |= (1 << ADSC);	// start conversion
	// wait until conversion is finished
	while ((ADCSRA & (1 << ADIF)) == 0)
		;
	unsigned short int result = ADC;
	ADCSRA |= (1 << ADIF);	// reset flag
	// voltage divider: VBAT - 27k - ADC - 10k - GND
	// results in a division factor of 3.7
	// voltage = ADC * 5V / 1024 * 3.7
	return (unsigned short int)(((unsigned long int)result * 18500UL) >> 10);
}


// fllt die beiden Shiftregister mit den bergebenen Daten (nur die unteren 12 Bit)
inline void fillShiftRegisters(unsigned short int data)
{
	for (unsigned short int  i=1; i<4096; i<<=1)
	{
		// Daten anlegen
		if (data & i)
			SHIFT_IN_HIGH();
		else
			SHIFT_IN_LOW();
		// steigende Flanke Shift-Clock
		SHIFT_CLK_HIGH();
		asm volatile ("nop");
		SHIFT_CLK_LOW();
	}
	
	// ein Impuls auf die Storage-Clock zum bernehmen der Daten
	STORAGE_CLK_HIGH();
	asm volatile ("nop");
	STORAGE_CLK_LOW();
}


// Timer 1 Compare Match Interrupt, steuert die LEDs entsprechend der Helligkeit/Farben in colorData an
ISR(TIM1_COMPA_vect, ISR_NOBLOCK)
{
	// Millisekunden mitzhlen
	static unsigned char timerCnt = 0;
	timerCnt++;
	
	if (timerCnt == 2)
	{
		unsigned long int timerMillisecondsTemp = timerMilliseconds;
		static unsigned char syncTimeout = 0;

		timerCnt = 0;
		
		if (!configuration.generateSyncPulse && (configuration.ppmEnable < 2))
		{
			// SCL pollen und wenn sich der Level gendert hat, Timeout zurcksetzen
			static unsigned char lastPinState = 0;
			unsigned char pinState = (PINA & (1 << PA4));
			if (pinState != lastPinState)
			{
				syncTimeout = 0;
				lastPinState = pinState;
				timerMillisecondsTemp += 5;
			}
		}
		
		static unsigned char syncDelay = 0;
		syncDelay++;
		if (syncDelay == 5)
		{
			if (configuration.generateSyncPulse)
			{
				timerMillisecondsTemp += 5;
				static unsigned char syncState = 0;
				syncState = ~syncState;
				if (syncState)
				{
					// Pin als Eingang schalten
					DDRA &= ~(1 << PA4);
				}
				else
				{
					// Pin als Ausgang schalten
					DDRA |= (1 << PA4);
				}
			}
			else
			{
				// prfen ob Sync-Impulse empfangen werden
				// falls nein: Variable selbst hochsetzen
				if (syncTimeout >= 3)
					timerMillisecondsTemp += 5;
				else
					syncTimeout++;
			}
			syncDelay = 0;
		}
		
		timerMilliseconds = timerMillisecondsTemp;
	}
	
	static unsigned char brightness = 0;
	unsigned short int leds = 0;
	// prfen, welche LEDs eingeschaltet sein mssen
	static COLOR_STATE colorStateTemp = RED;
	for (unsigned char i=0; i<12; i++)
	{
		if (colorData[i][colorStateTemp] > brightness)	// LED ist heller als aktuelle Helligkeit, also LED merken
			leds |= (1 << i);
	}
	FETS_OFF();	// alle FETs (fr die Farben) sperren
	fillShiftRegisters(leds);	// LEDs entsprechend der Helligkeiten einschalten
	// FET fr aktuelle Farbe einschalten und prfen ob zur nchsten Farbe gewechselt werden muss
	switch (colorStateTemp)
	{
		case RED:
			brightness++;
			if (brightness > 6)
				brightness = 0;
			RED_ON();
			colorStateTemp = GREEN;
		break;
		case GREEN:
			GREEN_ON();
			colorStateTemp = BLUE;
		break;
		case BLUE:
			BLUE_ON();
			colorStateTemp = RED;
		break;
	}
}


#ifdef USE_UART
// wandelt den String in eine Zahl um und liefert den Wert als unsigned long int zurck
unsigned long int parseNumber(unsigned char* string)
{
	unsigned char c;
	unsigned long int value = 0;
	while ((c = *string++))
	{
		if (c >= '0' && c <= '9')
			value = (value * 10) + c - '0';
		else
			break;
	}
	return value;
}


// gibt die aktuelle Versionsnummer aus
void printVersion(void)
{
	uartSendString_p(PSTR("\nVersion: "));
	uartSendString_p(versionNumber);
	uartTransmitByte('\n');
}


// gibt die aktuelle Einstellung zur Akku-Warnung aus
void printAkkuMode(void)
{
	uartSendString_p(PSTR("Akku-Warnung: "));
	if (configuration.akkuWarnung == 1)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);
	uartTransmitByte('\n');
}


// gibt die aktuellen Einstellungen zur PPM-Auswertung aus
void printPPM(void)
{
	uartSendString_p(PSTR("PPM-Auswertung: "));
	if (configuration.ppmEnable > 1)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);
	
	uartSendString_p(PSTR("\nPPM Min. [us]:"));
	uartSendNumber(((unsigned short int)configuration.ppmMinimum) << 5);
	uartSendString_p(PSTR("\nPPM Max. [us]:"));
	uartSendNumber(((unsigned short int)configuration.ppmMaximum) << 5);
	uartSendString_p(PSTR("\nPPM-Invertierung: "));
	if (configuration.ppmInvert == 1)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);
	
	uartSendString_p(ppmmodeText);
	uartSendString_p(PSTR(" Umschaltung: "));
	if (configuration.ppmModeSwitchOn == 1)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);

	for (unsigned char i=1; i<4; i++)
	{
		uartSendString_p(ppmmodeText);
		uartSendNumber(i);
		uartTransmitByte(':');
		uartSendNumber(configuration.ppmMode[i-1]);
	}
	uartTransmitByte('\n');
}


// gibt die aktuellen Akku-Werte fr leer und voll aus
void printAkkuValues(void)
{
	uartSendString_p(PSTR("Akku leer [mV]:"));
	uartSendNumber(configuration.akkuSpannungLeer);
	uartSendString_p(PSTR("\nAkku voll [mV]:"));
	uartSendNumber(configuration.akkuSpannungVoll);
	uartTransmitByte('\n');
}


// gibt aus ob die Akku-Zellenerkennung aktiviert ist
void printAkkuCellDetection(void)
{
	uartSendString_p(PSTR("Zellenerkennung: "));
	if (configuration.akkuCellDetection)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);
	uartTransmitByte('\n');
}


// gibt die aktuelle Blink-Geschwindigkeit aus, wenn der Akku leer ist
void printBlinkSpeed(void)
{
	uartSendString_p(PSTR("Blinkintervall [ms]:"));
	uartSendNumber(configuration.akkuBlinkInterval);
	uartTransmitByte('\n');
}


// gibt den bergebenen Text und die Farbe aus
//void printFarbe(const char* text, unsigned short int col)
//void printFarbe(const char* text, ColorStruct col)
void printFarbe(const char* text, unsigned char cols[3])
{
	uartSendString_p(PSTR("Akku Farbe "));
	uartSendString_p(text);
	for (unsigned char i=0; i<3; i++)
		uartTransmitByte('0' + cols[i]);
	uartTransmitByte('\n');
}


void printAkkuFarben(void)
{
	printFarbe(PSTR("leer: "), configuration.akkuFarbeBlink);
	printFarbe(PSTR("krit.: "), configuration.akkuFarbeKritisch);
	printFarbe(PSTR("voll: "), configuration.akkuFarbeOk);
}


void printLEDsCritical(void)
{
	uartSendString_p(PSTR("Akku LEDs krit.:"));
	uartSendNumber(configuration.ledsCritical);
	uartTransmitByte('\n');
}


// gibt die eingestellte I2C-Adresse aus
void printI2CAddress(void)
{
	uartSendString_p(PSTR("I2C-Adresse: 0x"));
	unsigned char val = configuration.I2CSlaveAddress >> 4;
	if (val <= 9)
		uartTransmitByte(val + '0');
	else
		uartTransmitByte(val-10+'A');
	val = configuration.I2CSlaveAddress & 15;
	if (val <= 9)
		uartTransmitByte(val + '0');
	else
		uartTransmitByte(val-10+'A');
	uartTransmitByte('\n');
}


// gibt den eingestellten Modus aus
void printMode(void)
{
	uartSendString_p(PSTR("Mode:"));
	unsigned char m;
	if (configuration.mode == IDLE)
		m = 0;
	else
		m = configuration.mode;
	uartSendNumber(configuration.mode);
	uartTransmitByte('\n');
}


// gibt aus ob ein Sync-Signal generiert wird oder nicht
void printSync(void)
{
	uartSendString_p(PSTR("Syncpuls: "));
	if (configuration.generateSyncPulse)
		uartSendString_p(textAn);
	else
		uartSendString_p(textAus);
	uartTransmitByte('\n');
}


// gibt die aktuelle Einstellung frs Dimmen aus
void printDimValue(void)
{
	uartSendString_p(PSTR("Dimmen:"));
	uartSendNumber(configuration.dimValue);
	uartTransmitByte('\n');
}


// gibt die Akkuspannung aus
void printAkkuVoltage(void)
{
	uartSendString_p(PSTR("Akkuspannung:"));
	uartSendNumber(akkuVoltageAverage);
	uartTransmitByte('\n');
}

#endif


void checkUART(void)
{
#ifdef USE_UART
	if (uartDataInBuffer())	// prfen ob Daten per UART empfangen wurden
	{
		unsigned long int timerMillisecondsTemp = timerMilliseconds;
		
		uartBuffer[uartBufferIndex] = uartReceiveByte();
		if (uartBuffer[uartBufferIndex] == '\n')	// auf Newline prfen
		{
			uartBuffer[uartBufferIndex] = 0;
			// Bufferinhalt auswerten
			if ((strlen((char*)uartBuffer) == 6) && (strncmp_P((char*)uartBuffer, PSTR("mode "), 5) == 0))
			{
				// Lichteffekt einstellen
				unsigned char mod = uartBuffer[5] - '0';
				if (mod <= 4)
				{
					configuration.mode = mod;
					// Mode im Flash speichern
					eeprom_write_block(&configuration.mode, &EEPromConfiguration.mode, sizeof(configuration.mode));
				}
				if (configuration.mode == PLAY_SCRIPT)
				{
					scriptBitIndex = 0;
					jumpBitIndex = 0;
					jumpLoopCount = 0;
					playDelay = timerMillisecondsTemp;
				}
				akkuBlinkDelay = timerMillisecondsTemp;
				printMode();
			}
			else if (strncmp_P((char*)uartBuffer, PSTR("info"), 5) == 0)
			{
				// Versionsnummer und weitere Einstellungen ausgeben
				printVersion();
				printMode();
				printSync();
				printDimValue();
				printAkkuMode();
				printAkkuValues();
				printAkkuCellDetection();
				printBlinkSpeed();
				printAkkuFarben();
				printLEDsCritical();
				printI2CAddress();
				printPPM();
				uartTransmitByte('\n');
			}
			else if (strncmp_P((char*)uartBuffer, PSTR("akku"), 5) == 0)
			{
				// Akku-Spannung ausgeben
				printAkkuVoltage();
			}
			else if ((strlen((char*)uartBuffer) > 9) && (strncmp_P((char*)uartBuffer, PSTR("akku"), 4) == 0) && (uartBuffer[5] == ' ') && (uartBuffer[4] == 'l' || uartBuffer[4] == 'v'))
			{
				// Schwellwert-Spannungen fr leeren bzw. vollen Akku einstellen (in Millivolt)
				unsigned long int value = parseNumber(uartBuffer+6);
				if (value >= 5000 && value <= 20000)
				{
					// Wert scheint plausibel zu sein
					if ((uartBuffer[4] == 'l') && (value < configuration.akkuSpannungVoll))
					{
						configuration.akkuSpannungLeer = value;
						eeprom_write_block(&configuration.akkuSpannungLeer, &EEPromConfiguration.akkuSpannungLeer, sizeof(configuration.akkuSpannungLeer));
					}
					else if ((uartBuffer[4] == 'v') && (value > configuration.akkuSpannungLeer))
					{
						configuration.akkuSpannungVoll = value;
						eeprom_write_block(&configuration.akkuSpannungVoll, &EEPromConfiguration.akkuSpannungVoll, sizeof(configuration.akkuSpannungVoll));
					}
					printAkkuValues();
				}
			}
			else if ((strlen((char*)uartBuffer) == 7) && (strncmp_P((char*)uartBuffer, PSTR("ppmm"), 4) == 0) && (uartBuffer[5] == ' '))
			{
				// PPM-Modus fr Mode-Wechsel einstellen
				unsigned long int value = parseNumber(uartBuffer+4);
				if ((value >= 1) && (value <= 3))
				{
					unsigned char mod = uartBuffer[6] - '0';
					if (mod <= 3)
					{
						// Wert scheint plausibel zu sein
						configuration.ppmMode[value-1] = mod;
						eeprom_write_block(&configuration.ppmMode[value-1], &EEPromConfiguration.ppmMode[value-1], sizeof(configuration.ppmMode[value-1]));
						printPPM();
					}
				}
			}
			else if ((strlen((char*)uartBuffer) > 7) && (strncmp_P((char*)uartBuffer, PSTR("ppm"), 3) == 0) && (uartBuffer[4] == ' ') && (uartBuffer[3] == 'l' || uartBuffer[3] == 'h'))
			{
				// PPM-Minimal bzw. Maximalwert
				unsigned long int value = parseNumber(uartBuffer+5);
				if (value >= 500 && value <= 2500)
				{
					value >>= 5;
					// Wert scheint plausibel zu sein
					if ((uartBuffer[3] == 'l') && (value < configuration.ppmMaximum))
					{
						configuration.ppmMinimum = (unsigned char)value;
						eeprom_write_block(&configuration.ppmMinimum, &EEPromConfiguration.ppmMinimum, sizeof(configuration.ppmMinimum));
					}
					else if ((uartBuffer[3] == 'h') && (value > configuration.ppmMinimum))
					{
						configuration.ppmMaximum = (unsigned char)value;
						eeprom_write_block(&configuration.ppmMaximum, &EEPromConfiguration.ppmMaximum, sizeof(configuration.ppmMaximum));
					}
					printPPM();
				}
			}
			else if ((strlen((char*)uartBuffer) == 8) && (strncmp_P((char*)uartBuffer, PSTR("ppminv "), 7) == 0))
			{
				// PPM-Signal ggf. invertieren
				if (uartBuffer[7] == '1')
					configuration.ppmInvert = 1;
				else
					configuration.ppmInvert = 0;
				// PPM-Invertierung im EEProm speichern
				eeprom_write_block(&configuration.ppmInvert, &EEPromConfiguration.ppmInvert, sizeof(configuration.ppmInvert));
				printPPM();
			}
			else if ((strlen((char*)uartBuffer) == 7) && (strncmp_P((char*)uartBuffer, PSTR("ppmsw "), 6) == 0))
			{
				// PPM-Signal zum Einstellen des Modus nutzen
				if (uartBuffer[6] == '1')
					configuration.ppmModeSwitchOn = 1;
				else
					configuration.ppmModeSwitchOn = 0;
				eeprom_write_block(&configuration.ppmModeSwitchOn, &EEPromConfiguration.ppmModeSwitchOn, sizeof(configuration.ppmModeSwitchOn));
				printPPM();
			}
			else if ((strlen((char*)uartBuffer) == 5) && (strncmp_P((char*)uartBuffer, PSTR("ppm "), 4) == 0))
			{
				// PPM-Auswertung ein/ausschalten
				if (uartBuffer[4] == '1')
				{
					configuration.ppmEnable = 2;
					configuration.generateSyncPulse = 0;
					eeprom_write_block(&configuration.generateSyncPulse, &EEPromConfiguration.generateSyncPulse, sizeof(configuration.generateSyncPulse));
				}
				else
					configuration.ppmEnable = 0;
				
				// PPM-Status im EEProm speichern
				eeprom_write_block(&configuration.ppmEnable, &EEPromConfiguration.ppmEnable, sizeof(configuration.ppmEnable));
				printPPM();
			}
			else if ((strlen((char*)uartBuffer) == 10) && (strncmp_P((char*)uartBuffer, PSTR("akkuwarn "), 9) == 0))
			{
				// Akku-Warnung bei leerem Akku unabhngig vom Lichteffekt anzeigen
				if (uartBuffer[9] == '1')
					configuration.akkuWarnung = 1;
				else
					configuration.akkuWarnung = 0;
				
				// AkkuWarnung im EEProm speichern
				eeprom_write_block(&configuration.akkuWarnung, &EEPromConfiguration.akkuWarnung, sizeof(configuration.akkuWarnung));
				printAkkuMode();
			}
			else if ((strlen((char*)uartBuffer) == 6) && (strncmp_P((char*)uartBuffer, PSTR("sync "), 5) == 0))
			{
				// Synchronisationsimpuls generieren
				if (uartBuffer[5] == '1')
				{
					configuration.generateSyncPulse = 1;
					PORTA &= ~(1 << PA4);	// Pin low schalten
					configuration.ppmEnable = 0;
					eeprom_write_block(&configuration.ppmEnable, &EEPromConfiguration.ppmEnable, sizeof(configuration.ppmEnable));
				}
				else
				{
					configuration.generateSyncPulse = 0;
					PORTA |= (1 << PA4);	// Pullup einschalten
				}
				
				// Sync-Einstellung im EEProm speichern
				eeprom_write_block(&configuration.generateSyncPulse, &EEPromConfiguration.generateSyncPulse, sizeof(configuration.generateSyncPulse));
				printSync();
			}
			else if ((strlen((char*)uartBuffer) == 9) && (strncmp_P((char*)uartBuffer, PSTR("celldet "), 8) == 0))
			{
				// Akku-Zellenerkennung ein-/ausschalten
				if (uartBuffer[8] == '1')
					configuration.akkuCellDetection = 1;
				else
					configuration.akkuCellDetection = 0;
				
				// Zellenerkennung im EEProm speichern
				eeprom_write_block(&configuration.akkuCellDetection, &EEPromConfiguration.akkuCellDetection, sizeof(configuration.akkuCellDetection));
				printAkkuCellDetection();
			}
			else if ((strlen((char*)uartBuffer) > 6) && (strncmp_P((char*)uartBuffer, PSTR("delay "), 6) == 0))
			{
				// Blink-Intervall setzen
				unsigned long int value = parseNumber(uartBuffer+6);
				if ((value >= 30) && (value <= 5000))
				{
					// Wert scheint plausibel zu sein, auf 5ms Granularitt bringen
					configuration.akkuBlinkInterval = (value/5) * 5;
					eeprom_write_block(&configuration.akkuBlinkInterval, &EEPromConfiguration.akkuBlinkInterval, sizeof(configuration.akkuBlinkInterval));
					//writeConfiguration();
					printBlinkSpeed();
				}
			}
			else if ((strlen((char*)uartBuffer) == 10) && (strncmp_P((char*)uartBuffer, PSTR("color"), 5) == 0) && (uartBuffer[6] == ' '))
			{
				// Farben einstellen (fr leeren Akku, kritische Spannung und ausreichend Spannung)
				unsigned char cols[3] = {uartBuffer[7] - '0', uartBuffer[8] - '0', uartBuffer[9] - '0'};
				/*
				unsigned char red = uartBuffer[7] - '0';
				unsigned char green = uartBuffer[8] - '0';
				unsigned char blue = uartBuffer[9] - '0';
				*/
				//if ((uartBuffer[7] >= '0') && (uartBuffer[7] <= '7') && (uartBuffer[8] >= '0') && (uartBuffer[8] <= '7') && (uartBuffer[9] >= '0') && (uartBuffer[9] <= '7'))
				//if ((red <= 7) && (green <= 7) && (blue <= 7))
				if ((cols[0] <= 7) && (cols[1] <= 7) && (cols[2] <= 7))
				{
					// Farb-Format ist in der Form colorX xxx mit x=0-7
					//ColorStruct color = {(uartBuffer[7]-'0'), (uartBuffer[8]-'0'), (uartBuffer[9]-'0')};
					//ColorStruct color = {red, green, blue};
					if (uartBuffer[5] == 'l')
					{
						// Akku-Spannung unter Schwellwert leer
						memcpy(configuration.akkuFarbeBlink, &cols, sizeof(cols));
						eeprom_write_block(&configuration.akkuFarbeBlink, &EEPromConfiguration.akkuFarbeBlink, sizeof(configuration.akkuFarbeBlink));
					}
					else if (uartBuffer[5] == 'k')
					{
						// Akku-Spannung kritisch
						memcpy(configuration.akkuFarbeKritisch, &cols, sizeof(cols));
						eeprom_write_block(&configuration.akkuFarbeKritisch, &EEPromConfiguration.akkuFarbeKritisch, sizeof(configuration.akkuFarbeKritisch));
					}
					else if (uartBuffer[5] == 'v')
					{
						// Akku voll
						memcpy(configuration.akkuFarbeOk, &cols, sizeof(cols));
						eeprom_write_block(&configuration.akkuFarbeOk, &EEPromConfiguration.akkuFarbeOk, sizeof(configuration.akkuFarbeOk));
					}
					printAkkuFarben();
				}
			}
			else if ((strlen((char*)uartBuffer) > 6) && (strncmp_P((char*)uartBuffer, PSTR("ledsk "), 6) == 0))
			{
				// LED-Anzahl einstellen, bei der die kritische Farbe im Akku-Bargraph-Modus verwendet wird
				unsigned long int value = parseNumber(uartBuffer+6);
				if ((value >= 1) && (value <= 11))
				{
					configuration.ledsCritical = (unsigned char)value;
					eeprom_write_block(&configuration.ledsCritical, &EEPromConfiguration.ledsCritical, sizeof(configuration.ledsCritical));
					//writeConfiguration();
					printLEDsCritical();
				}
			}
			else if ((strlen((char*)uartBuffer) == 5) && (strncmp_P((char*)uartBuffer, PSTR("dim "), 4) == 0))
			{
				// Dimmen einstellen
				unsigned char dim = uartBuffer[4] - '0';
				if (dim <= 6)
				//if (uartBuffer[4] >= '0' && uartBuffer[4] <= '6')
				{
					//configuration.dimValue = uartBuffer[4] - '0';
					configuration.dimValue = dim;
					eeprom_write_block(&configuration.dimValue, &EEPromConfiguration.dimValue, sizeof(configuration.dimValue));
				}
				printDimValue();
			}
			else if ((strlen((char*)uartBuffer) == 8) && (strncmp_P((char*)uartBuffer, PSTR("twi 0x"), 6) == 0))
			{
				// I2C-Adresse einstellen
				if (((uartBuffer[6] >= '0' && uartBuffer[6] <= '9') || (uartBuffer[6] >= 'A' && uartBuffer[6] <= 'F')) && ((uartBuffer[7] >= '0' && uartBuffer[7] <= '9') || (uartBuffer[7] >= 'A' && uartBuffer[7] <= 'F')))
				{
					// Parameter ist ein gltiger Hex-Wert
					unsigned char value = 0;
					for (unsigned char i=0; i<2; i++)
					{
						value <<= 4;
						if (uartBuffer[6+i] <= '9')
							value += (uartBuffer[6+i] - '0');
						else
							value += (10+(uartBuffer[6+i] - 'A'));
					}
					// Adressen 0 und >= 127 sind nicht erlaubt (7 Bit Adresse)
					if ((value > 0) && value < 127)
					{
						configuration.I2CSlaveAddress = value;
						// I2C-Adresse ins EEProm schreiben
						eeprom_write_block(&configuration.I2CSlaveAddress, &EEPromConfiguration.I2CSlaveAddress, sizeof(configuration.I2CSlaveAddress));
						printI2CAddress();
					}
				}
			}
			else if ((strlen((char*)uartBuffer) > 10) && (strncmp_P((char*)uartBuffer, PSTR("data "), 5) == 0))	// data 200FFFFF...    = data  adresse (3 zeichen) lnge (1 zeichen) + daten + check-byte (alles in hex)
			{
				// Adresse auswerten
				unsigned short int address = 0;
				for (unsigned char i=0; i<3; i++)
				{
					address <<= 4;
					if (uartBuffer[5+i] <= '9')
						address += (uartBuffer[5+i] - '0');
					else
						address += (10+(uartBuffer[5+i] - 'A'));
				}
				unsigned char check = (unsigned char)address;
				// Lnge auswerten
				unsigned char length;
				if (uartBuffer[8] <= '9')
					length = uartBuffer[8] - '0';
				else
					length = 10+(uartBuffer[8] - 'A');
				if (length > 15)
					length = 0;
				
				if ((address + length <= SCRIPT_DATA_SIZE) && (strlen((char*)uartBuffer) >= length*2 + 9 + 1))
				{
					if (configuration.mode == PLAY_SCRIPT)
					{
						playDelay = timerMillisecondsTemp + 3000;
						scriptBitIndex = 0;
						jumpBitIndex = 0;
						jumpLoopCount = 0;
					}
					unsigned char pos = 9;
					unsigned char data = 0;
					for (unsigned char p=0; p<length+1; p++)
					{
						for (unsigned char i=0; i<2; i++)
						{
							data <<= 4;
							if (uartBuffer[pos] <= '9')
								data += (uartBuffer[pos] - '0');
							else
								data += (10+(uartBuffer[pos] - 'A'));
							pos++;
						}
						if (p < length)
						{
							check ^= data;
							eeprom_write_byte(((unsigned char*)(&EEPromSkriptData)) + address + p, data);
						}
					}
					if (data == check)
						uartSendString_p(PSTR("ok\n"));
					else
						uartSendString_p(errorText);
				}
				else
				{
					// Skript zu gro
					uartSendString_p(errorText);
					// Damit kein unbrauchbares Skript im EEProm steht, definiertes Skript an den Anfang schreiben:
					// 1 144 3 128 0 (alle LEDs aus mit Dauer 500ms, dann Loop auf ersten Befehl)
					const unsigned char skript[5] = {1, 144, 3, 128, 0};
					for (unsigned char pos = 0; pos < 5; pos++)
						eeprom_write_byte(((unsigned char*)(&EEPromSkriptData)) + pos, skript[pos]);
				}
			}
			uartBufferIndex = 0;
		}
		else if ((uartBuffer[uartBufferIndex] != '\r') && (uartBufferIndex < UART_BUFFER_SIZE-1))
			uartBufferIndex++;
	}
#endif
}


void checkI2CSlave(void)
{
#ifdef USE_TWI_SLAVE
	if (I2CSlaveDataInReceiveBuffer())
	{
		twiBuffer[twiBufferIndex] = I2CSlaveReceiveByte();
		if ((twiBufferIndex == 1) && ((twiBuffer[0] == TWI_MODE) || (twiBuffer[0] == TWI_MODE_P)))
		{
			twiBufferIndex = 0;
			// Mode setzen
			if (twiBuffer[1] <= 4)
			{
				// Parameter plausibel
				configuration.mode = twiBuffer[1];
				if (twiBuffer[0] == TWI_MODE_P)
				{
					// Mode im Flash speichern
					eeprom_write_block(&configuration.mode, &EEPromConfiguration.mode, sizeof(configuration.mode));
				}
			}
		}
		else if ((twiBufferIndex == 2) && ((twiBuffer[0] == TWI_AKKU_LEER) || (twiBuffer[0] == TWI_AKKU_LEER_P) || (twiBuffer[0] == TWI_AKKU_VOLL) || (twiBuffer[0] == TWI_AKKU_VOLL_P)))
		{
			twiBufferIndex = 0;
			// Schwellwert fr leeren bzw. vollen Akku setzen
			unsigned short int value = (((unsigned short int)twiBuffer[1])<<8) | (unsigned short int)twiBuffer[2];
			if (value >= 5000 && value <= 20000)
			{
				// Wert scheint plausibel zu sein
				if (((twiBuffer[0] == TWI_AKKU_LEER) || (twiBuffer[0] == TWI_AKKU_LEER_P)) && (value < configuration.akkuSpannungVoll))
				{
					configuration.akkuSpannungLeer = value;
					if (twiBuffer[0] == TWI_AKKU_LEER_P)
					{
						eeprom_write_block(&configuration.akkuSpannungLeer, &EEPromConfiguration.akkuSpannungLeer, sizeof(configuration.akkuSpannungLeer));
					}
				}
				else if (((twiBuffer[0] == TWI_AKKU_VOLL) || (twiBuffer[0] == TWI_AKKU_VOLL_P)) && (value > configuration.akkuSpannungLeer))
				{
					configuration.akkuSpannungVoll = value;
					if (twiBuffer[0] == TWI_AKKU_VOLL_P)
					{
						eeprom_write_block(&configuration.akkuSpannungVoll, &EEPromConfiguration.akkuSpannungVoll, sizeof(configuration.akkuSpannungVoll));
					}
				}
			}
		}
		else if ((twiBufferIndex == 1) && ((twiBuffer[0] == TWI_AKKU_WARN) || (twiBuffer[0] == TWI_AKKU_WARN_P)))
		{
			twiBufferIndex = 0;
			// Akku-Warnung bei leerem Akku unabhngig vom Lichteffekt anzeigen
			if (twiBuffer[1] == 0)
				configuration.akkuWarnung = 0;
			else if (twiBuffer[1] == 1)
				configuration.akkuWarnung = 1;
			
			if (twiBuffer[0] == TWI_AKKU_WARN_P)
			{
				eeprom_write_block(&configuration.akkuWarnung, &EEPromConfiguration.akkuWarnung, sizeof(configuration.akkuWarnung));
			}
		}
		else if ((twiBufferIndex == 1) && (twiBuffer[0] == TWI_AKKU_CELLDETECTION_P))
		{
			twiBufferIndex = 0;
			// automatische Akku-Zellenanzahl-Erkennung
			if (twiBuffer[1] == 0)
				configuration.akkuCellDetection = 0;
			else if (twiBuffer[1] == 1)
				configuration.akkuCellDetection = 1;
			
			eeprom_write_block(&configuration.akkuCellDetection, &EEPromConfiguration.akkuCellDetection, sizeof(configuration.akkuCellDetection));
		}
		else if ((twiBufferIndex == 1) && ((twiBuffer[0] == TWI_DIM) || (twiBuffer[0] == TWI_DIM_P)))
		{
			twiBufferIndex = 0;
			// Dimm-Wert setzen
			if (twiBuffer[1] < 7)
			{
				// Wert scheint plausibel zu sein
				configuration.dimValue = twiBuffer[1];
				if (twiBuffer[0] == TWI_DIM_P)
				{
					eeprom_write_block(&configuration.dimValue, &EEPromConfiguration.dimValue, sizeof(configuration.dimValue));
				}
			}
		}
		else if ((twiBufferIndex == 18) && (twiBuffer[0] == TWI_DIRECT_COL))
		{
			twiBufferIndex = 0;
			// Farbe direkt setzen, 18 Bytes fr die Farben (4 Bit pro Farbe, 12 Bit pro LED)	// overhead, aber leichteres verarbeiten
			unsigned char index = 1;
			for (int i=0; i<12; i+=2)
			{
				colorData[i][RED] = twiBuffer[index] >> 4;
				colorData[i][GREEN] = twiBuffer[index++] & 15;
				colorData[i][BLUE] = twiBuffer[index] >> 4;
				
				colorData[i+1][RED] = twiBuffer[index++] & 15;
				colorData[i+1][GREEN] = twiBuffer[index] >> 4;
				colorData[i+1][BLUE] = twiBuffer[index++] & 15;
			}
			configuration.mode = IDLE;
		}
		else if ((twiBufferIndex == 0) && (twiBuffer[0] == TWI_READ_AKKU))
		{
			twiBufferIndex = 0;
			// Akku-Spannung auslesen
			I2CSlaveTransmitByte((unsigned char)(akkuVoltageAverage>>8));
			I2CSlaveTransmitByte((unsigned char)akkuVoltageAverage);
		}
		else if (twiBufferIndex < TWI_BUFFER_SIZE-1)
			twiBufferIndex++;
	}
#endif
}



// Hauptprogramm
void main(void) __attribute__ ((noreturn));

void main(void)
{
	initPorts();	// Ports initialisieren
	
	// initialisiert den AD-Wandler zum Messen der Akku-Spannung ber den Spannungsteiler
	ADCSRA = (1 << ADEN) | (0 << ADSC) | (0 << ADATE) | (0 << ADIF) | (0 << ADIE) | (1 << ADPS2) | (1 << ADPS1) | (0 << ADPS0);	// enable ADC, prescaler = 64 (~125 KHz clock frequency)
	ADCSRB = (0 << BIN) | (0 << ACME) | (0 << ADLAR) | (0 << ADTS2) | (0 << ADTS1) | (0 << ADTS0);	// free running mode
	ADMUX = (0 << REFS1) | (0 << REFS0) | (0  << MUX5) | (0 << MUX4) | (0 << MUX3) | (0 << MUX2) | (1 << MUX1) | (1 << MUX0);	// VCC as reference, PA3
	DIDR0 |= (1 << ADC3D);
	// Analog Comparator abschalten
	ACSR = (1 << ACD);
		
	FETS_OFF();	// alle FETS sperren
	ENABLE_LOW();	// Ausgnge der Shiftregister einschalten (active low)
	fillShiftRegisters(0);	// Shiftregister mit Nullen fllen = alles aus
	readConfiguration();	// Werte aus dem EEProm laden
	sei();	// Interrupts aktivieren
	
	// Timer zur Ansteuerung der LEDs starten
	TCNT1 = 0;
	OCR1A = 499;	// ~500us Timer Intervall
	TIMSK1 |= (1 <<  OCIE1A);
	TCCR1A = (0 << COM1A1) | (0 << COM1A0) | (0 << COM1B1) | (0 << COM1B0) | (0 << WGM11) | (0 << WGM10);	// CTC
	TCCR1B = (0 << ICNC1) | (0 << ICES1) | (0 << WGM13) | (1 << WGM12) | (0 << CS12) | (1 << CS11) | (0 << CS10);	// 1 MHz Clock
	TCCR1C = (0 << FOC1A) | (0 << FOC1B);

	#ifdef USE_UART
	initUART();	// UART initialisieren
	uartSendString_p(PSTR("\nLED Stripe"));
	printVersion();	
	#endif

	#ifdef USE_TWI_SLAVE
	I2CSlaveInit();	// I2C initialisieren
	#endif
	
	// ggf. Akku-Zellenzahl ermitteln und Schwellwerte anpassen
	delayms(100);	// warten bis sich die Spannung stabilisiert
	unsigned long int sum = 0;
	for (int i=0; i<5; i++)
		sum += getBatteryVoltage();	// Spannung ermitteln
	akkuVoltageAverage = sum / 5;
	if (configuration.akkuCellDetection)
	{
		// ggf. Schwellwerte anpassen
		unsigned char faktor = 0;
		if (akkuVoltageAverage > 16900)	// etwas Reserve beim Vergleich lassen
			faktor = 5;	// 5 Zellen
		else if (akkuVoltageAverage > 12700)	// etwas Reserve beim Vergleich lassen
			faktor = 4;	// 4 Zellen
		if (faktor != 0)
		{
			configuration.akkuSpannungLeer = (unsigned short int)(((unsigned long int)configuration.akkuSpannungLeer * faktor) / 3);
			configuration.akkuSpannungVoll = (unsigned short int)(((unsigned long int)configuration.akkuSpannungVoll * faktor) / 3);
		}
	}
	
	unsigned char blinkState = 1;
	unsigned char timerState = 0;
	unsigned char akkuBlinkTimeout = 0;
	unsigned long int ppmDelay = 0;
	
	while (1)
	{
		#ifdef USE_TWI_SLAVE
		checkI2CSlave();	// ggf. Datentransfer ber I2C abwickeln
		#endif
		
		#if USE_UART
		checkUART();	// ggf. Datentransfer ber UART abwickeln
		#endif
		
		unsigned long int timerMillisecondsTemp = timerMilliseconds;
		
		switch (configuration.ppmEnable)
		{
			default:
			break;
			case 0:
				PCMSK0 &= ~(1 << PCINT4);	// pin change interrupt fr PA4/SCK abschalten
				configuration.ppmEnable = 1;
			break;
			case 2:
				ppmDelay = timerMillisecondsTemp + 500;
				configuration.ppmEnable = 3;
			break;
			case 3:
				if ((ppmDelay - timerMillisecondsTemp) & 0x8000)
				{
					// PPM-Auswertung initialisieren
					ppmOk = 0;
					configuration.ppmEnable = 4;
					configuration.generateSyncPulse = 0;
					// SCK-Pin als Eingang schalten mit Pullup
					DDRA &= ~(1 << PA4);
					PORTA |= (1 << PA4);
					
					// Timer 0 einschalten
					TIMSK0 = 0;
					TCCR0A = 0;
					TCCR0B = (0 << FOC0A) | (0 << FOC0B) | (0 << WGM02) | (1 << CS02) | (0 << CS01) | (0 << CS00); // normal mode, clock = 8 MHz / 256 = 31250 Hz
					
					// Pin Change Interrupt fr SCK einschalten
					GIFR |= (1<<PCIF0);	// clear pin change interrupt flag
					PCMSK0 |= (1 << PCINT4);	// enable pin change for PA4
					GIMSK |= (1<<PCIE0);	// enable pin change interrupt
				}
			break;
			case 4:
				if (ppmOk)
				{
					unsigned char diff = configuration.ppmMaximum - configuration.ppmMinimum;
					ppmValue = ppmLow - ppmHigh;
					if (ppmValue <= configuration.ppmMinimum)
						ppmValue = 0;
					else if (ppmValue >= configuration.ppmMaximum)
						ppmValue = diff;
					else
						ppmValue = ppmValue - configuration.ppmMinimum;
					if (configuration.ppmInvert)
						ppmValue = diff - ppmValue;
					
					ppmOk = 0;
					
					if (configuration.ppmModeSwitchOn)
					{
						MODE newMode = configuration.mode;
						if (ppmValue <= diff/3)
							newMode = configuration.ppmMode[0];
						else if (ppmValue >= (diff - diff/3))
							newMode = configuration.ppmMode[2];
						else
							newMode = configuration.ppmMode[1];
						
						if (newMode != configuration.mode)
						{
							configuration.mode = newMode;
							if (configuration.mode == PLAY_SCRIPT)
							{
								scriptBitIndex = 0;
								jumpBitIndex = 0;
								jumpLoopCount = 0;
								playDelay = timerMillisecondsTemp;
							}
							akkuBlinkDelay = timerMillisecondsTemp;
						}
					}
				}
			break;
		}
		
		// Akku-Spannung regelmig messen
		static unsigned long int akkuTimer = 0;
		if ((akkuTimer - timerMillisecondsTemp) & 0x8000)
		{
			akkuTimer = timerMillisecondsTemp + 200;
			unsigned short int millivolts = getBatteryVoltage();	// Spannung ermitteln
			// Messwert etwas gltten
			if (akkuVoltageAverage == 0)
				akkuVoltageAverage = millivolts;
			else
				akkuVoltageAverage = (unsigned short int)((((unsigned long int)akkuVoltageAverage) * 5 + (unsigned long int)millivolts) / 6);
		}
		
		if ((configuration.mode == AKKU_SPANNUNG) || (configuration.akkuWarnung && (akkuVoltageAverage <= configuration.akkuSpannungLeer)) || (configuration.mode == PPM_BARGRAPH))
		{
			if (configuration.mode == IDLE)
				configuration.mode = OFF;
			else if (configuration.mode == PLAY_SCRIPT)
				akkuBlinkTimeout = 0;
			
			if ((akkuVoltageAverage > configuration.akkuSpannungLeer))
			{
				if ((akkuBlinkDelay - timerMillisecondsTemp) & 0x8000)
				{
					akkuBlinkDelay = timerMillisecondsTemp + 100;
					unsigned char leds;
					unsigned char cols[3];
					memcpy(cols, configuration.akkuFarbeOk, sizeof(cols));
					if (configuration.mode == PPM_BARGRAPH)
						leds = (unsigned char)(1 + ((unsigned short int)(ppmValue * 11)) / (configuration.ppmMaximum - configuration.ppmMinimum));
					else
					{
						leds = 1 + (akkuVoltageAverage - configuration.akkuSpannungLeer) / ((configuration.akkuSpannungVoll - configuration.akkuSpannungLeer) / 11);
						if (leds <= configuration.ledsCritical)
							memcpy(cols, configuration.akkuFarbeKritisch, sizeof(cols));
					}
					if (leds > 12)
						leds = 12;
					for (unsigned char i=0; i<leds; i++)
						memcpy((unsigned char*)colorData[i], cols, sizeof(cols));
					for (; leds<12; leds++)
						memset((unsigned char*)colorData[leds], 0, 3);
					blinkState = 2;
				}
			}
			else
			{
				akkuBlinkDelay = timerMillisecondsTemp;
				unsigned short int mod = (unsigned short int)(timerMillisecondsTemp % (configuration.akkuBlinkInterval*2));
				if (((mod == 0) && (blinkState == 0)) || (blinkState == 2))
				{
					// LEDs an
					blinkState = 1;
					// Akku leer, LEDs rot blinken lassen
					for (int i=0; i<12; i++)
						memcpy((unsigned char*)colorData[i], configuration.akkuFarbeBlink, sizeof(configuration.akkuFarbeBlink));
				}
				else if ((mod == configuration.akkuBlinkInterval) && (blinkState != 0))
				{
					// LEDs aus
					blinkState = 0;
					memset((unsigned char*)colorData, 0, 36);
				}
			}
		}
		else if (configuration.mode == OFF)
		{
			memset((unsigned char*)colorData, 0, 36);
			configuration.mode = IDLE;
		}
		else if (configuration.mode == TESTMODE)
		{
			static unsigned long int testDelay = 0;
			static unsigned char nextColor = 0;
			
			unsigned char mod = (unsigned char)(timerMillisecondsTemp % 100);
			if ((mod == 0) && (timerState == 0))
			{
				timerState = 1;
				if ((testDelay & 15) == 0)
				{
					unsigned char col[3] = {0, 0, 0};
					unsigned char brightness = 7 - configuration.dimValue;
					col[nextColor] = brightness;
					for (int i=0; i<12; i++)
						memcpy((unsigned char*)colorData[i], col, sizeof(col));
					
/*					for (int i=0; i<12; i++)
					{
						if (nextColor == 0)
							colorData[i][RED] = 7-configuration.dimValue;
						else
							colorData[i][RED] = 0;
						if (nextColor == 1)
							colorData[i][GREEN] = 7-configuration.dimValue;
						else
							colorData[i][GREEN] = 0;
						if (nextColor == 2)
							colorData[i][BLUE] = 7-configuration.dimValue;
						else
							colorData[i][BLUE] = 0;
					}*/
					nextColor++;
					if (nextColor > 2)
						nextColor = 0;
				}
				testDelay++;
			}
			else if (mod != 0)
				timerState = 0;
		}
		
		if (configuration.mode == PLAY_SCRIPT)
		{
			if (akkuBlinkTimeout == 1)
			{
				memcpy(colorData, colorDataScript, sizeof(colorData));
				blinkState = 2;
			}
			else if (akkuBlinkTimeout < 2)
				akkuBlinkTimeout++;
			
			if ((playDelay - timerMillisecondsTemp) & 0x8000)
			{
				unsigned short int duration = parseEEPromData(&scriptBitIndex);
				if (akkuBlinkTimeout >= 2)
					memcpy(colorData, colorDataScript, sizeof(colorData));
				if (duration > 10000)
					duration = 10000;
				playDelay += duration;
			}
		}
	}
	
}
