
#ifndef __MAIN_H__
#define __MAIN_H__

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/sleep.h>
#include <avr/eeprom.h>
#include <avr/sleep.h>

#include <string.h>

#include "uart.h"
#include "twi.h"


#define USE_UART		1

//#define USE_TWI_SLAVE	1


#if (USE_UART && USE_TWI_SLAVE)
	#error Cannot use UART and TWI slave at the same time!
#endif



#define DEFAULT_I2C_ADDRESS			0x69

#define DEFAULT_BLINK_INTERVAL			500


#define DEFAULT_AKKU_COLOR_EMPTY_RED			7
#define DEFAULT_AKKU_COLOR_EMPTY_GREEN		0
#define DEFAULT_AKKU_COLOR_EMPTY_BLUE			0

#define DEFAULT_AKKU_COLOR_CRITICAL_RED		7
#define DEFAULT_AKKU_COLOR_CRITICAL_GREEN	0
#define DEFAULT_AKKU_COLOR_CRITICAL_BLUE		0

#define DEFAULT_AKKU_COLOR_OK_RED				0
#define DEFAULT_AKKU_COLOR_OK_GREEN			7
#define DEFAULT_AKKU_COLOR_OK_BLUE				0

//#define DEFAULT_AKKU_COLOR_EMPTY		700		// RGB 0-7, hier: rot
//#define DEFAULT_AKKU_COLOR_CRITICAL	700		// RGB 0-7, hier: rot
//#define DEFAULT_AKKU_COLOR_OK			070		// RGB 0-7, hier: grn
#define DEFAULT_LEDS_CRITICAL				4			// ab 4 LEDs rote Farbe anzeigen

#define AKKU_SPANNUNG_VOLL			12600
#define AKKU_SPANNUNG_LEER			9000

#define UART_BUFFER_SIZE					42
#define TWI_BUFFER_SIZE					32

// I2C Kommandos
// schreibende Befehle
#define TWI_MODE								1		// Modus setzen, 1 Datenbyte (der zu setzende Modus), Gesamtlnge: 2 Bytes
#define TWI_MODE_P							2		// Modus setzen, 1 Datenbyte (der zu setzende Modus), Gesamtlnge: 2 Bytes
#define TWI_AKKU_LEER						5		// Schwellwert fr leeren Akku setzen, 2 Datenbytes (Schwellwert in Millivolts, MSB zuerst), Gesamtlnge: 3 Bytes
#define TWI_AKKU_LEER_P					6		// Schwellwert fr leeren Akku setzen, 2 Datenbytes (Schwellwert in Millivolts, MSB zuerst), Gesamtlnge: 3 Bytes
#define TWI_AKKU_VOLL						7		// Schwellwert fr vollen Akku setzen, 2 Datenbytes (Schwellwert in Millivolts, MSB zuerst), Gesamtlnge: 3 Bytes
#define TWI_AKKU_VOLL_P					8		// Schwellwert fr vollen Akku setzen, 2 Datenbytes (Schwellwert in Millivolts, MSB zuerst), Gesamtlnge: 3 Bytes
#define TWI_AKKU_WARN						9		// Warnung bei leerem Akku (de-)aktivieren, 1 Datenbyte (1 = Warnung an, 0 = Warnung aus), Gesamtlnge: 2 Bytes
#define TWI_AKKU_WARN_P					10	// Warnung bei leerem Akku (de-)aktivieren, 1 Datenbyte (1 = Warnung an, 0 = Warnung aus), Gesamtlnge: 2 Bytes
#define TWI_DIM									14	// Dimm-Wert setzen, 1 Datenbyte (0-7, 0 = nicht dimmen, 7 = maximale Dimmung), bezieht sich nur auf den Testmodus, Gesamtlnge: 2 Bytes
#define TWI_DIM_P								15	// Dimm-Wert setzen, 1 Datenbyte (0-7, 0 = nicht dimmen, 7 = maximale Dimmung), bezieht sich nur auf den Testmodus, Gesamtlnge: 2 Bytes
#define TWI_DIRECT_COL					16	// Direktes Setzen der LED-Farben, 18 Datenbytes (fr jede der 12 LEDs jeweils 4 Bit fr Rot, Grn und Blau (Helligkeit von 0-8)), Gesamtlnge: 19 Bytes
#define TWI_AKKU_CELLDETECTION_P	17	// Automatische Akku-Zellenzahl-Erkennung setzten, 1 Datenbyte (1 = Automatische Erkennung an, 0 = aus), Gesamtlnge: 2 Bytes
// lesende Befehle
#define TWI_READ_AKKU						101	// Akku-Spannung einlesen, 2 Datenbytes (Akkuspannung in Millivolt, MSB zuerst) als Antwort, Gesamtlnge: 1 Byte


// wartet die angegebene Zeit in Millisekunden
// XTAL_CPU muss entsprechend gesetzt sein
// (bentigt delay.h)
//extern void delay(unsigned int ms);
#define delayms(ms)               for (unsigned int i = ms; i > 0; --i)   \
                                                        _delay_loop_2(XTAL_CPU / 4000);


#define ENABLE_LOW()				PORTA &= ~(1 << PA0)
#define ENABLE_HIGH()				PORTA |= (1 << PA0)
#define STORAGE_CLK_LOW()		PORTA &= ~(1 << PA2)
#define STORAGE_CLK_HIGH()		PORTA |= (1 << PA2)
#define SHIFT_CLK_LOW()			PORTB &= ~(1 << PB2)
#define SHIFT_CLK_HIGH()			PORTB |= (1 << PB2)
#define SHIFT_IN_LOW()				PORTA &= ~(1 << PA7)
#define SHIFT_IN_HIGH()				PORTA |= (1 << PA7)

#define RED_OFF()						PORTB |= (1 << PB1)
#define RED_ON()						PORTB &= ~(1 << PB1)
#define BLUE_OFF()					PORTB |= (1 << PB0)
#define BLUE_ON()						PORTB &= ~(1 << PB0)
#define GREEN_OFF()					PORTA |= (1 << PA1)
#define GREEN_ON()					PORTA &= ~(1 << PA1)

#define FETS_OFF()					PORTA |= (1 << PA1); PORTB |= (1 << PB1) | (1 << PB0)


#define DISABLE_TIMER1_INTERRUPT()		TIMSK1 &= ~(1 <<  OCIE1A)
#define ENABLE_TIMER1_INTERRUPT()		TIMSK1 |= (1 <<  OCIE1A)


/*
Jeder Datensatz:
Befehl (3 Bit) + Dauer (10 Bit in 10ms, max 10s) = 13 Bit
(auer Loop)

Befehle:
- Farben aller LEDs setzen
	Alle Farbwerte: 3*3 * 12 Bit = 108 Bit
121 Bit total
- Scrollen links
	Neuer Farbwert fr freie LED: 3*3 Bit = 9 Bit
22 Bit total
- Scrollen rechts
	Neuer Farbwert fr freie LED: 3*3 Bit = 9 Bit
22 Bit total
- Rollen links
	keine Parameter
13 Bit total
- Rollen rechts
	keine Parameter
13 Bit total
- Setzen einzelner LEDs (fr Vernderungen)
	Anzahl LEDs: 3 Bit (maximal 8, 0=1, 1=2, 2=3...)
	fr jede LED:
		Position: 4 Bit + neuer Farbwert: 3*3 Bit = 9 Bit = 13 Bit
	(16 Bit fr eine LED, 29 Bit fr zwei LEDs, 42 Bit fr 3 LEDs..., 107 Bit fr 8 LEDs)
variable Bitzahl von 29 - 120 Bit
- alle LEDs auf eine Farbe setzen
	neuer Farbwert: 3*3 Bit = 9 Bit
22 Bit total
- Loop
	3 Bit Befehl
	12 Bit Sprungziel
	5 Bit Anzahl der Durchlufe
20 Bit total

*/

#define SCRIPT_ALL_SAME_COLOR		0
#define SCRIPT_SET_ALL_COLORS		1
#define SCRIPT_SCROLL_LEFT				2
#define SCRIPT_SCROLL_RIGHT			3
#define SCRIPT_ROLL_LEFT					4
#define SCRIPT_ROLL_RIGHT				5
#define SCRIPT_CHANGE_LEDS			6
#define SCRIPT_LOOP							7

#define SCRIPT_DATA_SIZE					450



typedef enum {RED = 0, GREEN = 1, BLUE = 2} COLOR_STATE;
typedef enum {OFF = 0, IDLE = 255, TESTMODE = 1, AKKU_SPANNUNG = 2, PLAY_SCRIPT = 3, PPM_BARGRAPH = 4} MODE;


typedef struct __attribute__((__packed__))
{
	MODE mode;
	unsigned short int akkuSpannungLeer;
	unsigned short int akkuSpannungVoll;
	unsigned char akkuWarnung;
	unsigned short int akkuBlinkInterval;
	unsigned char akkuFarbeBlink[3];
	unsigned char akkuFarbeKritisch[3];
	unsigned char akkuFarbeOk[3];
	unsigned char akkuCellDetection;
	unsigned char I2CSlaveAddress;
	unsigned char dimValue;
	unsigned char ledsCritical;
	unsigned char generateSyncPulse;
	unsigned char ppmEnable;
	unsigned char ppmInvert;
	unsigned char ppmMinimum;
	unsigned char ppmMaximum;
	unsigned char ppmModeSwitchOn;
	unsigned char ppmMode[3];
} Configuration;


extern Configuration configuration;

extern volatile unsigned char ppmLow;
extern volatile unsigned char ppmHigh;
extern volatile unsigned char ppmOk;

#endif
