// basierte ursprnglich auf Atmel Application Note AVR312: Using the USI module as a I2C slave
// angepasst fr den ATtiny84 mit kleinen nderungen und Ergnzungen

#ifndef __TWI_H__
#define __TWI_H__


#define TRUE                1
#define FALSE               0


// TWI slave
#define I2C_RX_BUFFER_SIZE  (32)
#define I2C_RX_BUFFER_MASK (I2C_RX_BUFFER_SIZE - 1)

#if (I2C_RX_BUFFER_SIZE & I2C_RX_BUFFER_MASK)
	#error TWI RX buffer size is not a power of 2
#endif

#define I2C_TX_BUFFER_SIZE  (8)
#define I2C_TX_BUFFER_MASK (I2C_TX_BUFFER_SIZE - 1)

#if (I2C_TX_BUFFER_SIZE & I2C_TX_BUFFER_MASK)
	#error TWI TX buffer size is not a power of 2
#endif

#define USI_SLAVE_CHECK_ADDRESS       		         		(0x00)
#define USI_SLAVE_SEND_DATA                    					(0x01)
#define USI_SLAVE_REQUEST_REPLY_FROM_SEND_DATA	(0x02)
#define USI_SLAVE_CHECK_REPLY_FROM_SEND_DATA		(0x03)
#define USI_SLAVE_REQUEST_DATA									(0x04)
#define USI_SLAVE_GET_DATA_AND_SEND_ACK				(0x05)



// TWI master defines
#define TWI_DELAY_T2		1.3	// >1.3us
#define TWI_DELAY_T4		0.6	// >0.6us

#define I2C_READ			1
#define I2C_WRITE			0

#define USI_TWI_NO_ACK_ON_DATA				0x05  // The slave did not acknowledge  all data
#define USI_TWI_NO_ACK_ON_ADDRESS		0x06  // The slave did not acknowledge  the address


// Defines for both, slave and master
#define DDR_USI				DDRA
#define PORT_USI			PORTA
#define PIN_USI				PINA
#define PORT_USI_SDA	PORTA6
#define PORT_USI_SCL		PORTA4
#define PIN_USI_SDA		PINA6
#define PIN_USI_SCL		PINA4



// TWI slave functions
void I2CSlaveFlushBuffers(void);
void I2CSlaveInit(void);
void I2CSlaveTransmitByte(unsigned char data);
unsigned char I2CSlaveReceiveByte(void);
unsigned char I2CSlaveDataInReceiveBuffer(void);


// TWI master functions
void I2CMasterInit(void);
unsigned char I2CMasterStartTransfer(unsigned char * , unsigned char);
unsigned char I2CMasterGetErrorState(void);


#endif
