// basiert auf Atmel Application Note AVR307: Half Duplex UART Using the USI Module
// angepasst fr den ATtiny84 mit kleinen nderungen und Ergnzungen

#ifndef _UART_H_
#define _UART_H_


#define BAUDRATE									9600


//********** USI UART Defines **********//

#define UART_RX_BUFFER_SIZE				8
#define UART_TX_BUFFER_SIZE				32

#define DATA_BITS									8
#define START_BIT									1	
#define STOP_BIT										1
#define HALF_FRAME								5

#define TIMER_PRESCALER						8

#define USI_COUNTER_MAX_COUNT			16
#define USI_COUNTER_SEED_TRANSMIT	(USI_COUNTER_MAX_COUNT - HALF_FRAME)
#define INTERRUPT_STARTUP_DELAY		(64 / TIMER_PRESCALER)
#define TIMER0_SEED								(((1UL * XTAL_CPU / BAUDRATE) / TIMER_PRESCALER)-3)

#if ( (( (XTAL_CPU / BAUDRATE) / TIMER_PRESCALER ) * 3/2) > (256 - INTERRUPT_STARTUP_DELAY) )
    #define INITIAL_TIMER0_SEED				((((XTAL_CPU / BAUDRATE) / TIMER_PRESCALER ) * 1/2)-1)
    #define USI_COUNTER_SEED_RECEIVE	(USI_COUNTER_MAX_COUNT - (START_BIT + DATA_BITS))
#else
    #define INITIAL_TIMER0_SEED				((((XTAL_CPU / BAUDRATE) / TIMER_PRESCALER ) * 3/2)-1)
    #define USI_COUNTER_SEED_RECEIVE	(USI_COUNTER_MAX_COUNT - DATA_BITS)
#endif

#define UART_RX_BUFFER_MASK ( UART_RX_BUFFER_SIZE - 1 )
#if ( UART_RX_BUFFER_SIZE & UART_RX_BUFFER_MASK )
    #error RX buffer size is not a power of 2
#endif

#define UART_TX_BUFFER_MASK ( UART_TX_BUFFER_SIZE - 1 )
#if ( UART_TX_BUFFER_SIZE & UART_TX_BUFFER_MASK )
    #error TX buffer size is not a power of 2
#endif

/* General defines */
#define TRUE                      1
#define FALSE                     0



#define uartDataInBuffer()		(UART_RxHead != UART_RxTail)

extern volatile unsigned char isUARTUsed;

extern volatile unsigned char UART_RxHead;
extern volatile unsigned char UART_RxTail;

void uartFlushBuffers(void);
void uartInitializeReceiver(void);
void uartInitializeTransmitter(void);
void uartTransmitByte(unsigned char data);
unsigned char uartReceiveByte(void);
//unsigned char uartDataInBuffer(void);
void initUART(void);
inline void uartDisableUART(void);

inline void uartSendString_p(const char *progmemPtr);
inline void uartSendString(const unsigned char *ptr);
inline void uartSendNumber(unsigned long int number);

#endif
